#include "configwin.h"
#include <stdio.h>

GtkWidget *update_freq_spin = NULL;
GtkWidget *scroll_len_spin = NULL;
GtkWidget *pad_red_spin = NULL;
GtkWidget *cdpath_box = NULL;
GtkWidget *time_fmt_box = NULL;
GtkWidget *date_fmt_box = NULL;

static void update_config() {
	UpdateFrequency = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(update_freq_spin));
	TitleScrollLength = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(scroll_len_spin));
	PadReduction= gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pad_red_spin));
	cdrom_mount_path = gtk_entry_get_text(GTK_ENTRY(cdpath_box));
	time_format = gtk_entry_get_text(GTK_ENTRY(time_fmt_box));
	date_format = gtk_entry_get_text(GTK_ENTRY(date_fmt_box));
}

static void close_config_window() {
	gtk_widget_hide(GTK_WIDGET(config_win));
}

/* Stolen from Stephen Watson's infowin.c */
static int trap_client_destroy(GtkWidget *widget, GdkEvent *event,
			      gpointer data) {
  gtk_widget_hide(GTK_WIDGET(data));
  return TRUE;
}

void config_win_show() {
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(update_freq_spin), 
	                          (gfloat) UpdateFrequency);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(scroll_len_spin), 
	                          (gfloat) TitleScrollLength);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pad_red_spin), 
	                          (gfloat) PadReduction);
	gtk_entry_set_text(GTK_ENTRY(cdpath_box), cdrom_mount_path);
	gtk_entry_set_text(GTK_ENTRY(time_fmt_box), time_format);
	gtk_entry_set_text(GTK_ENTRY(date_fmt_box), date_format);
	
	gtk_widget_show_all(GTK_WIDGET(config_win));
}

void config_win_init() {
	
	GtkWidget *vbox = NULL;
	GtkWidget *hbox = NULL;
	GtkWidget *btn = NULL;

	if (config_win != NULL) return;

	config_win = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_title(GTK_WINDOW(config_win), "ROX-MMS Config");
	/* Stolen from Stephen Watson's infowin.c */
	gtk_signal_connect(GTK_OBJECT(config_win), "delete_event", 
	                   GTK_SIGNAL_FUNC(trap_client_destroy), 
							 "WM destroy");

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(config_win), vbox);
	
	update_freq_spin = add_spin_entry(vbox, "Update frequency", 2000.0, 
	                              10.0, 10.0, (gfloat) UpdateFrequency);
	scroll_len_spin = add_spin_entry(vbox, "Title length", 25.0, 1.0, 
	                              1.0, (gfloat) TitleScrollLength);
	pad_red_spin = add_spin_entry(vbox, "Reduce pad by", 20.0, 1.0, 
	                              1.0, (gfloat) PadReduction);
	cdpath_box = add_entry_box(vbox, "CDROM mount path", cdrom_mount_path);
	time_fmt_box = add_entry_box(vbox, "Time format", time_format);
	date_fmt_box = add_entry_box(vbox, "Date format", date_format);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	btn = gtk_button_new_with_label("Apply");
	gtk_signal_connect(GTK_OBJECT(btn), "clicked", 
	                   GTK_SIGNAL_FUNC(update_config), config_win);
	gtk_box_pack_start(GTK_BOX(hbox), btn, TRUE, TRUE, 0);
	btn = gtk_button_new_with_label("Close");
	gtk_signal_connect(GTK_OBJECT(btn), "clicked", 
	                   GTK_SIGNAL_FUNC(close_config_window), config_win);
	gtk_box_pack_start(GTK_BOX(hbox), btn, TRUE, TRUE, 0);
	
}

GtkWidget *add_spin_entry(GtkWidget *vbox, gchar *text, gfloat maxval, 
                          gfloat step, gfloat page, gfloat initval) {
	
	GtkWidget *hbox = NULL;
	GtkWidget *spin = NULL;
	GtkWidget *label = NULL;
	GtkObject *spin_adj = NULL;

	/* Set up the hbox and add it to the vbox. */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_set_homogeneous(GTK_BOX(hbox), TRUE);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	/* Create the label and add it to the hbox. */
	label = gtk_label_new(text);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);

	/* We need to make an adjustment object for the spin button. */
	spin_adj = gtk_adjustment_new(initval, 1.0, maxval, step, page, 1.0);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(spin_adj), step, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	
	return spin;
	
}

GtkWidget *add_entry_box(GtkWidget *vbox, gchar *label, gchar *text) {

	GtkWidget *hbox = NULL;
	GtkWidget *textbox = NULL;
	GtkWidget *box_label = NULL;

	/* Set up the hbox and add it to the vbox. */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_set_homogeneous(GTK_BOX(hbox), TRUE);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	/* Create the label and add it to the hbox. */
	box_label = gtk_label_new(label);
	gtk_label_set_justify(GTK_LABEL(box_label), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX(hbox), box_label, TRUE, TRUE, 0);
	/* Add the entry and set the default text. */
	textbox = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(textbox), text);
	gtk_box_pack_start(GTK_BOX(hbox), textbox, TRUE, TRUE, 0);

	return textbox;

}
